/*---------------------------------------------------------------------------\
| Project:			Generic LV2 PIC18F458 CAN node firmware
|
| File:				atv.c
|
| Description:		Mainline routines.
|
| Global Routines:	Main()
|
| Local Routines:	None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include <pic18.h>    	// Include for PICC-18
#include "pic18fxx8.h"	// Register definitions for the PIC18F458
#include "piccore\core.h"	// Includes for the project (Globals + misc)
#include <string.h>
#include <stdio.h>

#if CORE_VERSION != 4
#error HEY! The core version has changed! Make SURE your interfaces match!
#endif

void Initialize (void);
void InitializeVariables (void);
void process_UTC();
void process_LatLong();
void process_Height();
void process_Lock();

/* message currently being processed */
CanMessage_t curMsg;

// Note that the GPSID only really uses the top five bits to determine it's a GPS
// message - the bottom 6 bits tell us what kind of GPS message it.

#define GPSID 0b00011000000

typedef enum {
	UTC_YEAR,
	UTC_MONTH,
	UTC_DAY,
	UTC_HOURS,
	UTC_MINUTES,
	UTC_SECONDS,
} utc;

#define FLOATFMT "%+1.8f"

/*---------------------------------------------------------------------------\
| Function:		main
|
| Purpose:		Run all Mainline code
|
| Discussion:	Doesn't do much, huh?
|
| Expects:		Uh let's see... nothing.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void main (void)
{
    uint32_t i;

    // Initialize everything and its brother
    Initialize();

    // Wait for BOB board to not be an idiot
    for(i=0; i < 1000000; i++);

    // PSAS Initialize Screen 
    printf("{C0000PSAS");
    printf("{C0001LV2b");
	
    // Hang out for awhile to display PSAS Initialize Screen
    for(i=0; i < 2000000; i++);
	
    // Clear screen
    printf("{A");


    // Send a hello CAN packet

    curMsg.header.all = 0b0101010000000000;
    SendCanMessage(&curMsg);
	
	while(TRUE)
	{
		if(canMessageAvailable)
		{
			GetCanMessage(&curMsg);

			switch(curMsg.header.all >> 5)
			{
				case GPSID | 0b00010:				//UTC
					process_UTC();
					break;
				case GPSID | 0b00011:				//Latitude and Longitude
					process_LatLong();
					break;
				case GPSID | 0b00100:				//Height
					process_Height();
					break;
				case GPSID | 0b00001:				//lock
					process_Lock();
					break;
			}
		}
	}
}




/*---------------------------------------------------------------------------\
| Function:		Initialize
|
| Purpose:		Initialize everything. Really.
|
| Discussion:	Did I mention initialization?
|
| Expects:		to be called after a RESET - eg, registers reset to known
|				values and and interrupts off.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Initialize (void)
{

	/*
		EVENTUALLY: Check for a reset: If there hasn't been a reset,
		then something has gone horribly wrong. Do a SOFTWARE RESET
		here to put everything in a known state.
	*/

	// Setting up the CAN bus peripheral: We're passing constants
	// to the initialization routine so that we can just set the
	// initialize initialization :) here.

	// Set CAn buffer 0 (the high priority buffer) to handle all the ATVID message
	// and CAN buffer 1 (Low priority) to handle the GPSID messages
	
	const CanHeader_t canMaskB0 =     0b11111111111;	// Set acceptance mask0
	const CanHeader_t canMaskB1 =     0b11111111111;    // Set acceptance mask1
	
	const CanHeader_t CanFilterB0F0 = 0b00001000000;    // Set Can Buffer 0 Filter 0
	const CanHeader_t CanFilterB0F1 = 0b00001000001;    // Set Can Buffer 0 Filter 1
	
	const CanHeader_t CanFilterB1F2 = 0b00011000001;    // B0 F2 = GPSID + Lock
	const CanHeader_t CanFilterB1F3 = 0b00011000010;    // B0 F3 = GPSID + UTC
	const CanHeader_t CanFilterB1F4 = 0b00011000011;    // B0 F4 = GPSID + Lat/Lon
	const CanHeader_t CanFilterB1F5 = 0b00011000100;    // B0 F5 = GPSID + Height

	// Get the outputs stabilized FIRST.

	InitializePorts(0b00000000, // latchA
	                0b00000000, // directionA (1 = in, 0 = out)
	                0b00000000, // latchB
	                0b00001000, // directionB (1 = in, 0 = out) 
        			// CAN peripheral requires TRISB -> RB2 = 0, RB3 = 1
	                0b00000000, // latchC
	                0b00000000, // directionC (1 = in, 0 = out)
	                0b11111110, // latchD
	                0b00000000, // directionD (1 = in, 0 = out)
	                0b00000000, // latchE
	                0b00000000, // directionE (1 = in, 0 = out)
	                (PORTS_PORTB_PULLUPS_OFF
	                | PORTS_INT0_RISING_EDGE
	                | PORTS_INT1_RISING_EDGE) );

	// Turn on serial transmitter and receiver. Turn on receiver interrupts
	// but leave the transmitter interrupts off (we'll turn them on when
	// there's actual data to send over the port). Make both interrupts
	// low priority, and se the bit rate to 9600bps.

	InitializeSerial (	SERIAL_TRANSMIT_ENABLE
						| SERIAL_TRANSMIT_DISABLE_INTS
						| SERIAL_RECEIVE_ENABLE
						| SERIAL_RECEIVE_DISABLE_INTS
						| SERIAL_BITRATE_9600 );

	// Turn on timer2, HP interrupts @ 1KHz

	InitializeTimer2 (	TIMER2_DISABLE
						| TIMER2_DISABLE_INTS
						| TIMER2_1KHZ );

	// Turn on CAN: See const variable declarations above
	// for the values. Note that as soon as we initialize the
	// the bus, we can start to receive messages, so be sure
	// that interrupts are enabled VERY SOON after this call
	// (No more than about a  message length, which at top
	// speed is about 1/22KHz = 45us which at 20MHz is only
	// 250 instructions.

	InitializeCan (	CAN_BITRATE_1MBPS,
					canMaskB0,			// Mask RXB0
					canMaskB1,			// Mask RXB1
					CanFilterB0F0,		// Filter RXB0 F0
					CanFilterB0F1,		// Filter RXB0 F1
					CanFilterB1F2,		// Filter RXB1 F2
					CanFilterB1F3,		// Filter RXB1 F3
					CanFilterB1F4,		// Filter RXB1 F4
					CanFilterB1F5);		// Filter RXB1 F5

	// Initialize interrupts

	InitializeInterrupts();

	// DO NO FURTHER INITIALIZATION AFTER INTERRUPTS SINCE WE'RE OFF AND
	// RUNNING ONCE THEY'RE ENABLED.
}






void putch(char ch)
{
	SendSerialByte(ch);
}	

static uint32_t getData32(uint8_t index)
{
	index <<= 2;
	return (uint8_t) curMsg.data[index] << 24 |
	       (uint8_t) curMsg.data[index + 1] << 16 |
	       (uint8_t) curMsg.data[index + 2] << 8 |
	       (uint8_t) curMsg.data[index + 3];
}

uint8_t getDate(utc c)
{
	return curMsg.data[c];
}

void process_UTC()
{
	printf("{C1110");
	printf("%d:%d:%d %d/%d/0%d",
		getDate(UTC_HOURS),
		getDate(UTC_MINUTES),
		getDate(UTC_SECONDS),
		getDate(UTC_MONTH),
		getDate(UTC_DAY),
		getDate(UTC_YEAR));
}

/* Format to display data to screen
0000000000111111111122222222
0123456789012345678901234567
                         nnL

+01.01234567   +012.01234567
+01234.0M  hh:mm:ss mm/dd/yy
*/

void process_LatLong()
{
	// Latitude
	printf("{C0009");
	printf(FLOATFMT, (float)getData32(0) / 100000000.0);

	// Longitude
	printf("{C1509");
	printf(FLOATFMT, (float)getData32(1) / 100000000.0);
}

void process_Height()
{
	printf("{C0010");
	printf(FLOATFMT, (float)getData32(0) / 100.0);
	printf("m");
}

void process_Lock()
{
	/* prints # of sats seen */
	printf("{C2500");
	printf("%d", curMsg.data[0]);
	
	/* both [1] and [2] must be zero to be truely locked */
	if(curMsg.data[1] || curMsg.data[2] == 0)
		printf("{C2700{T6EH");
}
