/*---------------------------------------------------------------------------\
| Project:          Generic LV2 PIC18F458 CAN node firmware
|
| File:             interrupts.c
|
| Description:      Interrupt Handlers.
|
| Global Routines:  InitializeInterrupts
|
| Local Routines:   None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include <piccore.h>   // Includes for the project (includes other includes!)


//------------------------------------------------------------------------
//  Local function prototypes
//------------------------------------------------------------------------


/*---------------------------------------------------------------------------\
| Function:     HighPriorityISR
|
| Purpose:      Handle all of the high priority interrupts
|
| Discussion:   Checks to see what interrupted us. Why all this weird #ifdef
|               stuff? Because the PIC can't due recursive calls, and we have
|               an error function ("ThrowWarning") with a mainline, high
|               priority, and low priority flavors. In order for the compiler
|               to not yell at us, we need to figure out which interrupts
|               are which priority and #ifdef the correct error functions.
|               Ugly, but it's the best we can do (for now).
|
|               Why check to see if the interrupt is enabled? Because
|               any interrupt gets us here, we need to check to see
|               which interrupt needs servicing. But how can we tell if
|               if the interrupt flag is set but the interrupt is disabled?
|               We have to manually check. Stupid, but true.
|
| Expects:      all interrupts to be disabled.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

// DECLARE THIS FUNCTION TO BE THE HIGH PRIORITY INTERRUPT SERVICE ROUTINE
// USING THE "interrupt" KEYWORD.


void interrupt HighPriorityISR (void)
{
    static bit hpTest;

    hpTest = FALSE;

    #if INTS_TIMER2_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_TIMER2_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    if ( Timer2InterruptEnabled == TRUE
         && Timer2InterruptPending == TRUE )
    {
        timer2_isr();
        hpTest = TRUE;
    }
    #endif

    #if (INTS_INT0_ISR_USED == INTS_CORE_ISR_ENABLED)   // NB: INT0 is high priority only
    if ( Int0InterruptEnabled == TRUE
         && Int0InterruptPending == TRUE )
    {
        int0_isr();
        hpTest = TRUE;
    }
    #endif

    #if INTS_INT1_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_INT1_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    if ( Int1InterruptEnabled == TRUE
         && Int1InterruptPending == TRUE )
    {
        int1_isr();
        hpTest = TRUE;
    }
    #endif

    #if INTS_ADC_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_ADC_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    if ( AdcInterruptEnabled == TRUE
         && AdcInterruptPending == TRUE )
    {
        adc_isr();
        hpTest = TRUE;
    }
    #endif

    #if INTS_SERIAL_TX_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_SERIAL_TX_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    if ( SerialTxInterruptEnabled == TRUE
         && SerialTxInterruptPending == TRUE )
    {
        SerialTxIsr();
        hpTest = TRUE;
    }
    #endif

    #if INTS_SERIAL_RX_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_SERIAL_RX_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    if ( SerialRxInterruptEnabled == TRUE
         && SerialRxInterruptPending == TRUE )
    {
        SerialRxIsr();
        hpTest = TRUE;
    }
    #endif

    #if INTS_CAN_TX2_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_CAN_TX2_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    if ( CanTxb2InterruptEnabled == TRUE
         && CanTxb2InterruptPending == TRUE )
    {
        CanTx2Isr();
        hpTest = TRUE;
    }
    #endif

    #if INTS_CAN_TX1_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_CAN_TX1_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    if ( CanTxb1InterruptEnabled == TRUE
         && CanTxb1InterruptPending == TRUE )
    {
        CanTx1Isr();
        hpTest = TRUE;
    }
    #endif

    #if INTS_CAN_TX0_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_CAN_TX0_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    if ( CanTxb0InterruptEnabled == TRUE
         && CanTxb0InterruptPending == TRUE )
    {
        CanTx0Isr();
        hpTest = TRUE;
    }
    #endif

    #if INTS_CAN_RX0_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_CAN_RX0_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    if ( CanRxb0InterruptEnabled == TRUE
         && CanRxb0InterruptPending == TRUE )
    {
        CanRx0Isr();
        hpTest = TRUE;
    }
    #endif

    #if INTS_CAN_RX1_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_CAN_RX1_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    if ( CanRxb1InterruptEnabled == TRUE
         && CanRxb1InterruptPending == TRUE )
    {
        CanRx1Isr();
        hpTest = TRUE;
    }
    #endif

    // Throw a warning if we don't know why were interrupted.

    if (hpTest == FALSE)
        ThrowWarning_HP(WARN_ISR_UNKNOWN_HPI);
}



/*---------------------------------------------------------------------------\
| Function:     LowPriorityISR
|
| Purpose:      Handle all of the low prioirty interrupts
|
| Discussion:   Checks to see what interrupted us.
|
| Expects:      Low prioirty interrupts disabled but high prioirty may still
|               be enabled.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

// DECLARE THIS FUNCTION TO BE THE LOW PRIOIRTY INTERRUPT SERVICE ROUTINE
// USING THE "interrupt low-priority" KEYWORDS.

void interrupt low_priority LowPriorityISR (void)
{
    static bit lpTest;

    lpTest = FALSE;

     #if INTS_TIMER2_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_TIMER2_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
     if ( Timer2InterruptEnabled == TRUE
          && Timer2InterruptPending == TRUE )
     {
         timer2_isr();
         lpTest = TRUE;
     }
     #endif

     #if INTS_INT1_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_INT1_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
     if ( Int1InterruptEnabled == TRUE
          && Int1InterruptPending == TRUE )
     {
         int1_isr();
         lpTest = TRUE;
     }
     #endif

     #if INTS_ADC_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_ADC_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
     if ( AdcInterruptEnabled == TRUE
          && AdcInterruptPending == TRUE )
     {
         adc_isr();
         lpTest = TRUE;
     }
     #endif

     #if INTS_SERIAL_TX_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_SERIAL_TX_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
     if ( SerialTxInterruptEnabled == TRUE
          && SerialTxInterruptPending == TRUE )
     {
         SerialTxIsr();
         lpTest = TRUE;
     }
     #endif

     #if INTS_SERIAL_RX_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_SERIAL_RX_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
     if ( SerialRxInterruptEnabled == TRUE
          && SerialRxInterruptPending == TRUE )
     {
         SerialRxIsr();
         lpTest = TRUE;
     }
     #endif

     #if INTS_CAN_TX2_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_CAN_TX2_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
     if ( CanTxb2InterruptEnabled == TRUE
          && CanTxb2InterruptPending == TRUE )
     {
         CanTx2Isr();
         lpTest = TRUE;
     }
     #endif

     #if INTS_CAN_TX1_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_CAN_TX1_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
     if ( CanTxb1InterruptEnabled == TRUE
          && CanTxb1InterruptPending == TRUE )
     {
         CanTx1Isr();
         lpTest = TRUE;
     }
     #endif

     #if INTS_CAN_TX0_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_CAN_TX0_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
     if ( CanTxb0InterruptEnabled == TRUE
          && CanTxb0InterruptPending == TRUE )
     {
         CanTx0Isr();
         lpTest = TRUE;
     }
     #endif

     #if INTS_CAN_RX0_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_CAN_RX0_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
     if ( CanRxb0InterruptEnabled == TRUE
          && CanRxb0InterruptPending == TRUE )
     {
         CanRx0Isr();
         lpTest = TRUE;
     }
     #endif

     #if INTS_CAN_RX1_ISR_USED == INTS_CORE_ISR_ENABLED && INTS_CAN_RX1_INT_PRIORITY == INTS_LOW_PRIORITY_INTERRUPT
     if ( CanRxb1InterruptEnabled == TRUE
          && CanRxb1InterruptPending == TRUE )
     {
         CanRx1Isr();
         lpTest = TRUE;
     }
    #endif

    // Throw a warning if we don't know why were interrupted.

    if (lpTest == FALSE)
        ThrowWarning_LP(WARN_ISR_UNKNOWN_LPI);
}





/*---------------------------------------------------------------------------\
| Function:     InitializeInterrupts
|
| Purpose:      Initialize the PIC's interrupts.
|
| Discussion:   For each interrupt source (timer 2, ADC, CAN, etc), we're
|               going to choose the interrupt priority (low or high) and
|               whether we're going to enable or disable the interrupts.
|               We have to use compiler macros to do all this because the
|               the compiler will complain at us in other areas and heck,
|               it produces tight code (not that we need it in the
|               initialization routine). Once we're done setting priorities
|               and enables, we turn the interrupts on and we're off!
|
| Expects:      --
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void InitializeInterrupts (void)
{
    // Turn on "high and low" priorities, not old peripheral/global interrupts
    // This will always be the case for all the nodes.

    EnablePrioritizedInts();

    // Set the interrupt priority and enable or disable the UART interrupts


    #if INTS_SERIAL_TX_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    SerialTxPrioritySetting = HIGH_PRIORITY_INT;
    #else
    SerialTxPrioritySetting = LOW_PRIORITY_INT;
    #endif

    #if INTS_SERIAL_RX_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    SerialRxPrioritySetting = HIGH_PRIORITY_INT;
    #else
    SerialRxPrioritySetting = LOW_PRIORITY_INT;
    #endif

    // *** Turn on ints and away we go! ***

    EnableHighPriorityInts();
    EnableLowPriorityInts();
}

